import pytest
from datetime import datetime, timedelta
import urlparse
from decimal import Decimal

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.core.wsgi import get_wsgi_application
from webtest import TestApp

from django.test import Client

from lingo.models import Regie, BasketItem, Transaction, RemoteItem

pytestmark = pytest.mark.django_db

client = Client()

@pytest.fixture
def regie():
    try:
        regie = Regie.objects.get(slug='test')
    except Regie.DoesNotExist:
        regie = Regie()
        regie.label = 'Test'
        regie.slug = 'test'
        regie.description = 'test'
        regie.payment_min_amount = Decimal(4.5)
        regie.service = 'dummy'
        regie.service_options = {'siret': '1234'}
        regie.save()
    return regie

@pytest.fixture
def user():
    try:
        user = User.objects.get(username='admin')
    except User.DoesNotExist:
        user = User.objects.create_user('admin', email=None, password='admin')
    return user

def login(username='admin', password='admin'):
    resp = client.post('/login/', {'username': username, 'password': password})
    assert resp.status_code == 302

def test_payment_min_amount(regie, user):
    items = {'item1': {'amount': '1.5', 'source_url': '/item/1'},
             'item2': {'amount': '2.4', 'source_url': '/item/2'}
    }
    b_items = []
    for subject, details in items.iteritems():
        b_item = BasketItem.objects.create(user=user, regie=regie,
                                    subject=subject, **details)
        b_items.append(b_item.pk)
    login()
    resp = client.post(reverse('lingo-pay'), {'item': b_items, 'regie': regie.pk})
    assert resp.status_code == 403

def test_successfull_items_payment(regie, user):
    items = {'item1': {'amount': '10.5', 'source_url': '/item/1'},
             'item2': {'amount': '42', 'source_url': '/item/2'},
             'item3': {'amount': '100', 'source_url': '/item/3'},
             'item4': {'amount': '354', 'source_url': '/item/4'}
    }
    b_items = []
    for subject, details in items.iteritems():
        b_item = BasketItem.objects.create(user=user, regie=regie,
                                    subject=subject, **details)
        b_items.append(b_item.pk)
    login()
    resp = client.post(reverse('lingo-pay'), {'item': b_items, 'regie': regie.pk})
    assert resp.status_code == 302
    location = resp.get('location')
    assert 'dummy-payment' in location
    parsed = urlparse.urlparse(location)
    # get return_url and transaction id from location
    qs = urlparse.parse_qs(parsed.query)
    args = {'transaction_id': qs['transaction_id'][0], 'signed': True,
            'ok': True, 'reason': 'Paid'}
    # simulate backend callback call
    resp = client.get(qs['return_url'][0], args)
    assert resp.status_code == 200
